@@ -80,7 +80,8 @@ class TestMember(base.BaseAPITest):
         self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
         auth_strategy = self.conf.conf.api_settings.get('auth_strategy')
         self.conf.config(group='api_settings', auth_strategy=constants.TESTING)
-        with mock.patch.object(octavia.common.context.Context, 'project_id',
+        with mock.patch.object(octavia.common.context.RequestContext,
+                               'project_id',
                                self.project_id):
             override_credentials = {
                 'service_user_id': None,
@@ -88,7 +89,7 @@ class TestMember(base.BaseAPITest):
                 'is_admin_project': True,
                 'service_project_domain_id': None,
                 'service_project_id': None,
-                'roles': ['load-balancer_member'],
+                'roles': ['load-balancer_member', 'member'],
                 'user_id': None,
                 'is_admin': False,
                 'service_user_domain_id': None,
@@ -110,7 +111,8 @@ class TestMember(base.BaseAPITest):
         self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
         auth_strategy = self.conf.conf.api_settings.get('auth_strategy')
         self.conf.config(group='api_settings', auth_strategy=constants.TESTING)
-        with mock.patch.object(octavia.common.context.Context, 'project_id',
+        with mock.patch.object(octavia.common.context.RequestContext,
+                               'project_id',
                                uuidutils.generate_uuid()):
             response = self.get(self.member_path.format(
                 member_id=api_member.get('id')), status=403).json
@@ -183,7 +185,8 @@ class TestMember(base.BaseAPITest):
         self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
         auth_strategy = self.conf.conf.api_settings.get('auth_strategy')
         self.conf.config(group='api_settings', auth_strategy=constants.TESTING)
-        with mock.patch.object(octavia.common.context.Context, 'project_id',
+        with mock.patch.object(octavia.common.context.RequestContext,
+                               'project_id',
                                self.project_id):
             override_credentials = {
                 'service_user_id': None,
@@ -191,7 +194,7 @@ class TestMember(base.BaseAPITest):
                 'is_admin_project': True,
                 'service_project_domain_id': None,
                 'service_project_id': None,
-                'roles': ['load-balancer_member'],
+                'roles': ['load-balancer_member', 'member'],
                 'user_id': None,
                 'is_admin': False,
                 'service_user_domain_id': None,
@@ -229,7 +232,8 @@ class TestMember(base.BaseAPITest):
         self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
         auth_strategy = self.conf.conf.api_settings.get('auth_strategy')
         self.conf.config(group='api_settings', auth_strategy=constants.TESTING)
-        with mock.patch.object(octavia.common.context.Context, 'project_id',
+        with mock.patch.object(octavia.common.context.RequestContext,
+                               'project_id',
                                None):
             override_credentials = {
                 'service_user_id': None,
@@ -268,7 +272,8 @@ class TestMember(base.BaseAPITest):
         self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
         auth_strategy = self.conf.conf.api_settings.get('auth_strategy')
         self.conf.config(group='api_settings', auth_strategy=constants.TESTING)
-        with mock.patch.object(octavia.common.context.Context, 'project_id',
+        with mock.patch.object(octavia.common.context.RequestContext,
+                               'project_id',
                                uuidutils.generate_uuid()):
             response = self.get(self.members_path, status=403)
         self.conf.config(group='api_settings', auth_strategy=auth_strategy)
@@ -347,10 +352,10 @@ class TestMember(base.BaseAPITest):
         members = self.get(self.members_path, params={
             'fields': ['id', 'address']}).json
         for member in members['members']:
-            self.assertIn(u'id', member)
-            self.assertIn(u'address', member)
-            self.assertNotIn(u'name', member)
-            self.assertNotIn(u'monitor_address', member)
+            self.assertIn('id', member)
+            self.assertIn('address', member)
+            self.assertNotIn('name', member)
+            self.assertNotIn('monitor_address', member)
 
     def test_get_one_fields_filter(self):
         member1 = self.create_member(
@@ -360,10 +365,10 @@ class TestMember(base.BaseAPITest):
         member = self.get(
             self.member_path.format(member_id=member1.get('id')),
             params={'fields': ['id', 'address']}).json.get(self.root_tag)
-        self.assertIn(u'id', member)
-        self.assertIn(u'address', member)
-        self.assertNotIn(u'name', member)
-        self.assertNotIn(u'monitor_address', member)
+        self.assertIn('id', member)
+        self.assertIn('address', member)
+        self.assertNotIn('name', member)
+        self.assertNotIn('monitor_address', member)
 
     def test_get_all_filter(self):
         mem1 = self.create_member(self.pool_id,
@@ -515,7 +520,8 @@ class TestMember(base.BaseAPITest):
         auth_strategy = self.conf.conf.api_settings.get('auth_strategy')
         self.conf.config(group='api_settings', auth_strategy=constants.TESTING)
 
-        with mock.patch.object(octavia.common.context.Context, 'project_id',
+        with mock.patch.object(octavia.common.context.RequestContext,
+                               'project_id',
                                self.project_id):
             override_credentials = {
                 'service_user_id': None,
@@ -523,7 +529,7 @@ class TestMember(base.BaseAPITest):
                 'is_admin_project': True,
                 'service_project_domain_id': None,
                 'service_project_id': None,
-                'roles': ['load-balancer_member'],
+                'roles': ['load-balancer_member', 'member'],
                 'user_id': None,
                 'is_admin': False,
                 'service_user_domain_id': None,
@@ -563,7 +569,8 @@ class TestMember(base.BaseAPITest):
         auth_strategy = self.conf.conf.api_settings.get('auth_strategy')
         self.conf.config(group='api_settings', auth_strategy=constants.TESTING)
 
-        with mock.patch.object(octavia.common.context.Context, 'project_id',
+        with mock.patch.object(octavia.common.context.RequestContext,
+                               'project_id',
                                self.project_id):
             api_member = self.create_member(
                 self.pool_id, '192.0.2.1', 80, status=403)
@@ -584,7 +591,7 @@ class TestMember(base.BaseAPITest):
         self.set_object_status(self.pool_repo, pool1_id,
                                provisioning_status=constants.ERROR)
         api_member = self.create_member(pool1_id, '192.0.2.1', 80, status=409)
-        ref_msg = 'Pool %s is immutable and cannot be updated.' % pool1_id
+        ref_msg = f'Pool {pool1_id} is immutable and cannot be updated.'
         self.assertEqual(ref_msg, api_member.get('faultstring'))
 
     # TODO(rm_work) Remove after deprecation of project_id in POST (R series)
@@ -682,7 +689,7 @@ class TestMember(base.BaseAPITest):
         # Order matters here
         provider_creates += provider_updates
 
-        mock_provider.assert_called_once_with(u'noop_driver',
+        mock_provider.assert_called_once_with('noop_driver',
                                               mock_driver.member_batch_update,
                                               self.pool_id, provider_creates)
 
@@ -725,7 +732,7 @@ class TestMember(base.BaseAPITest):
             provider_dict['name'] = None
             provider_members.append(driver_dm.Member(**provider_dict))
 
-        mock_provider.assert_called_once_with(u'noop_driver',
+        mock_provider.assert_called_once_with('noop_driver',
                                               mock_driver.member_batch_update,
                                               self.pool_id, provider_members)
 
@@ -785,7 +792,7 @@ class TestMember(base.BaseAPITest):
         req_dict = [member1, member2, member5, member6]
         body = {self.root_tag_list: req_dict}
         path = self.MEMBERS_PATH.format(pool_id=self.pool_id)
-        path = "{}?additive_only=True".format(path)
+        path = f"{path}?additive_only=True"
         self.put(path, body, status=202)
         returned_members = self.get(
             self.MEMBERS_PATH.format(pool_id=self.pool_id)
@@ -827,7 +834,7 @@ class TestMember(base.BaseAPITest):
         provider_creates += provider_updates
         provider_creates += provider_ignored
 
-        mock_provider.assert_called_once_with(u'noop_driver',
+        mock_provider.assert_called_once_with('noop_driver',
                                               mock_driver.member_batch_update,
                                               self.pool_id, provider_creates)
 
@@ -878,7 +885,7 @@ class TestMember(base.BaseAPITest):
             del provider_dict['subnet_id']
             provider_members.append(driver_dm.Member(**provider_dict))
 
-        mock_provider.assert_called_once_with(u'noop_driver',
+        mock_provider.assert_called_once_with('noop_driver',
                                               mock_driver.member_batch_update,
                                               self.pool_id, provider_members)
 
@@ -906,6 +913,38 @@ class TestMember(base.BaseAPITest):
             m_subnet_exists.assert_called_once_with(
                 member1['subnet_id'], context=mock.ANY)
 
+    @mock.patch('octavia.api.drivers.driver_factory.get_driver')
+    @mock.patch('octavia.api.drivers.utils.call_provider')
+    def test_update_members_member_duplicate(
+            self, mock_provider, mock_get_driver):
+        mock_driver = mock.MagicMock()
+        mock_driver.name = 'noop_driver'
+        mock_get_driver.return_value = mock_driver
+        subnet_id = uuidutils.generate_uuid()
+
+        member1 = {'address': '192.0.2.1', 'protocol_port': 80,
+                   'project_id': self.project_id, 'subnet_id': subnet_id}
+
+        req_dict = [member1]
+        body = {self.root_tag_list: req_dict}
+        path = self.MEMBERS_PATH.format(pool_id=self.pool_id)
+        self.put(path, body, status=202)
+
+        self.set_lb_status(self.lb_id)
+
+        # Same member (same address and protocol_port) updated twice in the
+        # same PUT request
+        member1 = {'address': '192.0.2.1', 'protocol_port': 80,
+                   'project_id': self.project_id, 'subnet_id': subnet_id,
+                   'name': 'member1'}
+        member2 = {'address': '192.0.2.1', 'protocol_port': 80,
+                   'project_id': self.project_id, 'subnet_id': subnet_id,
+                   'name': 'member2'}
+
+        req_dict = [member1, member2]
+        body = {self.root_tag_list: req_dict}
+        self.put(path, body, status=400)
+
     @mock.patch('octavia.api.drivers.driver_factory.get_driver')
     @mock.patch('octavia.api.drivers.utils.call_provider')
     def test_update_members_subnet_not_found(
@@ -964,7 +1003,7 @@ class TestMember(base.BaseAPITest):
                  rm['protocol_port'],
                  rm['provisioning_status']), expected_members)
 
-        mock_provider.assert_called_once_with(u'noop_driver',
+        mock_provider.assert_called_once_with('noop_driver',
                                               mock_driver.member_batch_update,
                                               self.pool_id, provider_members)
 
@@ -1130,7 +1169,8 @@ class TestMember(base.BaseAPITest):
         self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
         auth_strategy = self.conf.conf.api_settings.get('auth_strategy')
         self.conf.config(group='api_settings', auth_strategy=constants.TESTING)
-        with mock.patch.object(octavia.common.context.Context, 'project_id',
+        with mock.patch.object(octavia.common.context.RequestContext,
+                               'project_id',
                                self.project_id):
             override_credentials = {
                 'service_user_id': None,
@@ -1138,7 +1178,7 @@ class TestMember(base.BaseAPITest):
                 'is_admin_project': True,
                 'service_project_domain_id': None,
                 'service_project_id': None,
-                'roles': ['load-balancer_member'],
+                'roles': ['load-balancer_member', 'member'],
                 'user_id': None,
                 'is_admin': False,
                 'service_user_domain_id': None,
@@ -1183,7 +1223,8 @@ class TestMember(base.BaseAPITest):
         self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
         auth_strategy = self.conf.conf.api_settings.get('auth_strategy')
         self.conf.config(group='api_settings', auth_strategy=constants.TESTING)
-        with mock.patch.object(octavia.common.context.Context, 'project_id',
+        with mock.patch.object(octavia.common.context.RequestContext,
+                               'project_id',
                                self.project_id):
             member_path = self.member_path_listener.format(
                 member_id=api_member.get('id'))
@@ -1310,7 +1351,8 @@ class TestMember(base.BaseAPITest):
         self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
         auth_strategy = self.conf.conf.api_settings.get('auth_strategy')
         self.conf.config(group='api_settings', auth_strategy=constants.TESTING)
-        with mock.patch.object(octavia.common.context.Context, 'project_id',
+        with mock.patch.object(octavia.common.context.RequestContext,
+                               'project_id',
                                self.project_id):
             override_credentials = {
                 'service_user_id': None,
@@ -1318,7 +1360,7 @@ class TestMember(base.BaseAPITest):
                 'is_admin_project': True,
                 'service_project_domain_id': None,
                 'service_project_id': None,
-                'roles': ['load-balancer_member'],
+                'roles': ['load-balancer_member', 'member'],
                 'user_id': None,
                 'is_admin': False,
                 'service_user_domain_id': None,
@@ -1359,7 +1401,8 @@ class TestMember(base.BaseAPITest):
         self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
         auth_strategy = self.conf.conf.api_settings.get('auth_strategy')
         self.conf.config(group='api_settings', auth_strategy=constants.TESTING)
-        with mock.patch.object(octavia.common.context.Context, 'project_id',
+        with mock.patch.object(octavia.common.context.RequestContext,
+                               'project_id',
                                self.project_id):
             self.delete(self.member_path_listener.format(
                 member_id=api_member.get('id')), status=403)
@@ -1391,7 +1434,7 @@ class TestMember(base.BaseAPITest):
         member_path = self.MEMBERS_PATH.format(
             pool_id=bad_pool_id) + '/' + api_member['id']
         result = self.delete(member_path, status=404).json
-        ref_msg = 'Member %s not found.' % api_member['id']
+        ref_msg = f"Member {api_member['id']} not found."
         self.assertEqual(ref_msg, result.get('faultstring'))
 
     @mock.patch('octavia.api.drivers.utils.call_provider')
